using System;
using System.Collections;
using System.Xml;
using System.Diagnostics;


namespace FigureList
{
	public class FigureCollection : ArrayList
	{

		public FigureCollection()
		{
		}

		public new Figure this [int i]
		{
			get { return (Figure) base[i]; }
			set { base[i] = value; }		
		}

		public override string ToString()
		{
			string s = "";
			for (int i = 0; i < base.Count; i++)
			{
				Figure f = (Figure) base[i];
				s += String.Format("{0} {0}\n", i, f.Id);
			}
			return base.ToString ();
		}

		public void SerializeToXMLFile(string fileName)
		{
			XmlTextWriter xw;
			// Create a new XmlTextWriter.
			xw = new XmlTextWriter(fileName, System.Text.Encoding.UTF8);
			xw.Formatting = Formatting.Indented;
			xw.Indentation = 1;
			xw.IndentChar = '\t';

			xw.WriteStartDocument();

			// Write the beginning of the "data" element. This is 
			// the opening tag to our data 

			xw.WriteStartElement("FigureList");
			for (int i = 0; i < base.Count; i++)
			{
				Figure f = (Figure) base[i];
				xw.WriteStartElement("Figure");
				xw.WriteAttributeString("Id", f.Id);
				xw.WriteElementString("Type", f.Type);
				xw.WriteElementString("Size", XmlConvert.ToString(f.Size));
				xw.WriteElementString("Color", XmlConvert.ToString(f.ColorNr));
				xw.WriteEndElement();
			}

			xw.WriteEndElement();
			xw.WriteEndDocument();

			xw.Close();

		}

		public void SerializeFromXmlFile (string fileName)
		{
			XmlTextReader xr = new XmlTextReader(fileName);
			xr.WhitespaceHandling = WhitespaceHandling.None;

			base.Clear();

			string msg;
			string id = "";
			string type = "Undefined";
			Figure f = null;
			string sRead = "";

			xr.MoveToContent();
			while (xr.Read())
			{
				if (xr.NodeType == XmlNodeType.Element)
				{
					string name = xr.Name;
					switch (xr.Name) 
					{
						case "Figure":
							id = xr.GetAttribute("Id");
							break;
						case "Type":
							type = xr.ReadString();
							switch (type)
							{
								case "Circle":
									f = new Circle();
									break;
								case "Square":
									f = new Square();
									break;
								case "Triangle":
									f = new Triangle();
									break;
								default:
									f = null;
									break;
							}
							f.Id = id;
							base.Add(f);
							break;
						case "Size":
							sRead = xr.ReadString();
							f.Size = (float) XmlConvert.ToDouble(sRead);
							break;
						case "Color":
							sRead = xr.ReadString();
							f.ColorNr = XmlConvert.ToInt32(sRead);
							break;
						default:
							sRead = "";
							break;
					}
					msg = String.Format("{0} {1}", name, sRead);
					Debug.WriteLine(msg);
				}
			}
			xr.Close();
		}

	}
}
